
#include "image_io.h"

int writeImage(char* filename, int width, int height, unsigned char *buffer, char* title) {
  int code = 0;
  FILE *fp;
  png_structp png_ptr;
  png_infop info_ptr = NULL;
  png_bytep row = NULL;
  
  // Open file for writing (binary mode)
  fp = fopen(filename, "wb");
  if (fp == NULL) {
    fprintf(stderr, "Could not open file %s for writing\n", filename);
    code = 1;
    goto finalise;
  }
  
  // Initialize write structure
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (png_ptr == NULL) {
    fprintf(stderr, "Could not allocate write struct\n");
    code = 1;
    goto finalise;
  }

  // Initialize info structure
  info_ptr = png_create_info_struct(png_ptr);
  if (info_ptr == NULL) {
    fprintf(stderr, "Could not allocate info struct\n");
    code = 1;
    goto finalise;
  }
  
  // Setup Exception handling
  if (setjmp(png_jmpbuf(png_ptr))) {
    fprintf(stderr, "Error during png creation\n");
    code = 1;
    goto finalise;
  }
  
  png_init_io(png_ptr, fp);
  
  // B&W image
  // Write header (8 bit gray)
  png_set_IHDR(png_ptr, info_ptr, width, height,
	       8, PNG_COLOR_TYPE_GRAY, PNG_INTERLACE_NONE,
	       PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
  
  // Set title
  if (title != NULL) {
    png_text title_text;
    title_text.compression = PNG_TEXT_COMPRESSION_NONE;
    title_text.key = "Title";
    title_text.text = title;
    png_set_text(png_ptr, info_ptr, &title_text, 1);
  }
  
  png_write_info(png_ptr, info_ptr);
  
  // Allocate memory for one row (1 byte per pixel)
  row = (png_bytep) malloc(width * sizeof(png_byte));
  
  // Write image data
  int x, y;
  for (y=0 ; y<height ; y++) {
    for (x=0 ; x<width ; x++) {
      row[x] = (buffer[y*width + x]);
    }
    png_write_row(png_ptr, row);
  }
  
  // End write
  png_write_end(png_ptr, NULL);

 finalise:
  if (fp != NULL) fclose(fp);
  if (info_ptr != NULL) png_free_data(png_ptr, info_ptr, PNG_FREE_ALL, -1);
  if (info_ptr != NULL){ free(info_ptr); info_ptr = NULL; }
  if (png_ptr != NULL) png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
  if(png_ptr != NULL){ free(png_ptr); png_ptr = NULL; }
  if (row != NULL) free(row);
  
  return code;
}

unsigned char *read_mask(char *mask_fname, int *mask_width, int *mask_height){
  *mask_width = -1;
  *mask_height = -1;
  //return NULL;
  return read_image(mask_fname, mask_width, mask_height);
}

unsigned char *read_image(char *fname, int *width, int *height){

  long y;
  register long x;
 
  MagickBooleanType status;
  //  MagickPixelPacket pixel;
  MagickWand *image_wand;
  PixelIterator *iterator;
  PixelWand **pixels;

  unsigned long img_width;

  unsigned char *img_mask = NULL;
  int i = 0;

  MagickWandGenesis();
  image_wand=NewMagickWand();
  status = MagickReadImage(image_wand,fname);
  if(status == MagickFalse)
    return NULL;

  *width = (int)MagickGetImageWidth(image_wand);
  *height = (int)MagickGetImageHeight(image_wand);

  img_mask = (unsigned char*)malloc((*width)*(*height)*sizeof(unsigned char));

  iterator=NewPixelIterator(image_wand);

  for(y=0; y < (long) MagickGetImageHeight(image_wand); y++) {
    pixels=PixelGetNextIteratorRow(iterator,&img_width);
    
    if(pixels == (PixelWand **) NULL)
      break;
    for(x=0; x < (long) img_width; x++){

      i = y*img_width+x;

      if(PixelGetRedQuantum(pixels[x]) == MaxRGB &&
	 PixelGetGreenQuantum(pixels[x]) == 0 &&
	 PixelGetBlueQuantum(pixels[x]) == 0){
	img_mask[(*width)*(*height)-i-1] = 0;
      } else {
	img_mask[(*width)*(*height)-i-1] = 1;
      }
    }
  }
  iterator=DestroyPixelIterator(iterator);
  image_wand=DestroyMagickWand(image_wand);
  MagickWandTerminus();

  return img_mask;

}
